@@ -119,7 +119,7 @@ void VMThread::create() {
 
   if (UsePerfData) {
     // jvmstat performance counters
-    Thread* THREAD = Thread::current();
+    JavaThread* THREAD = JavaThread::current(); // For exception macros.
     _perf_accumulated_vm_operation_time =
                  PerfDataManager::create_counter(SUN_THREADS, "vmOperationTime",
                                                  PerfData::U_Ticks, CHECK);
@@ -395,7 +395,7 @@ void VMThread::inner_execute(VM_Operation* op) {
   _cur_vm_operation = op;
 
   HandleMark hm(VMThread::vm_thread());
-  EventMark em("Executing %s VM operation: %s", prev_vm_operation != NULL ? "nested" : "", op->name());
+  EventMarkVMOperation em("Executing %sVM operation: %s", prev_vm_operation != NULL ? "nested " : "", op->name());
 
   log_debug(vmthread)("Evaluating %s %s VM operation: %s",
                        prev_vm_operation != NULL ? "nested" : "",
@@ -521,7 +521,9 @@ void VMThread::execute(VM_Operation* op) {
   SkipGCALot sgcalot(t);
 
   // JavaThread or WatcherThread
-  t->check_for_valid_safepoint_state();
+  if (t->is_Java_thread()) {
+    t->as_Java_thread()->check_for_valid_safepoint_state();
+  }
 
   // New request from Java thread, evaluate prologue
   if (!op->doit_prologue()) {
